@@ -63,6 +63,9 @@ public:
         return sym_ == key.sym_ && states_ == key.states_ && code_ == key.code_;
     }
 
+    /// Check if key is not same;
+    bool operator!=(const Key &key) const { return !operator==(key); }
+
     /// Check if current key match the key.
     bool check(const Key &key) const;
 
@@ -73,9 +76,52 @@ public:
         return check(Key(sym, states));
     }
 
-    /// Check if key is digit key.
+    /**
+     * Check if current key is a key release of given modifier only key.
+     *
+     * This is a very specialized check for modifier release case.
+     * And it's designed to handle modifier only key.
+     *
+     * For example, if Alt+Shift_L is pressed, then the following release key of
+     * this event can be either: Alt+Shift+Shift_L, or Alt+Shift+Meta_{L,R}.
+     * This is because: Alt -> Meta_{L,R}, if alt is released first, then it
+     * will produce Alt+Shift+Meta_{L,R}. If shift is released first, then it
+     * will produce Alt+Shift+Shift_L.
+     *
+     * Return false if key is not a modifier.
+     */
+    bool isReleaseOfModifier(const Key &key) const;
+
+    /**
+     * Check if key is digit key or keypad digit key.
+     *
+     * Since 5.0.20, it will also return true for keypad digit key.
+     * @return Whether the key is a digit key without modifier.
+     */
     bool isDigit() const;
 
+    /**
+     * Return the value of digit key.
+     *
+     * The key must not have modifier.
+     * For example, FcitxKey_7 will return 7.
+     *
+     * @return value of digit key. If the key is not a digit key, it returns -1.
+     * @since 5.0.20
+     */
+    int digit() const;
+
+    /**
+     * Return index when using digit key for selection.
+     *
+     * The return value will return in the order of 1234567890.
+     *
+     * @return index of digit key. If the key is not a digit key, it returns -1.
+     * @since 5.0.20
+     * @see digit
+     */
+    int digitSelection(KeyStates states = KeyStates()) const;
+
     /// Check if key is upper case.
     bool isUAZ() const;
 
@@ -98,6 +144,9 @@ public:
     /// Check if states has modifier.
     bool hasModifier() const;
 
+    /// Check if states has virtual bit
+    bool isVirtual() const;
+
     /// \brief Normalize a key, usually used when key is from frontend.
     ///
     /// states will be filtered to have only ctrl alt shift and super.
@@ -169,7 +218,7 @@ public:
     /// Check the current key against a key list.
     /// \see fcitx::Key::check
     template <typename Container>
-    bool checkKeyList(const Container &c) {
+    bool checkKeyList(const Container &c) const {
         return std::find_if(c.begin(), c.end(), [this](const Key &toCheck) {
                    return check(toCheck);
                }) != c.end();
@@ -179,7 +228,7 @@ public:
     /// \return Returns the matched key index or -1 if there is no match.
     /// \see fcitx::Key::check
     template <typename Container>
-    int keyListIndex(const Container &c) {
+    int keyListIndex(const Container &c) const {
         size_t idx = 0;
         for (auto &toCheck : c) {
             if (check(toCheck)) {
